#  see rtgf.tex, rtgf.pdf for explanation
import sys,os
from numpy import *
import numpy.linalg as la

Nb,Ne = map(int,sys.argv[1:3])
vc0 = float(sys.argv[3])


def simp():
   wt=array([0.0]*Nb)
   for i in range(Nb):
     wt[i] = (i % 2+1)*2./3.
   wt[0] = 1./6.
   wt[Nb-1] = wt[0]
   return wt
   
wt = simp()

def ff():
  f = array([0.0]*Nb)
  denom = float(Nb-1)
  for i in range(Nb):
    f[i] = 1-(i/denom)**2         
  return f
  
f = ff() 
  
def pnice(mat,strng) :
  print strng
  rows,columns =shape(mat)
  for i in range(rows) :
    ss=''
    for j in range(columns):
       t1=mat[i,j]
       if abs(t1) < 0.0001 :
         ss +='   0   '
       else :
         ss +=' %6.3f' % mat[i,j]
    print ss
  print

emax=1.0  # single-particle energy spacing
de = emax/(Nb-1)
dt=0.25*pi  # time step 
Nt=256  # number of time steps
vc = vc0/Nb
ss=' %4i %4i %8.5f %5.2f %5.3f %4i' % (Nb,Ne,de,vc,dt,Nt)
print 'Nb,Ne,de,vc,dt,Nt', ss
ham=array([[0.0]*Nb]*Nb )
for i1 in range(Nb) :
  ham[i1,i1] = i1*de
  for i2 in range(Nb) :
    ham[i1,i2] += vc
#    ham[i1,i2] += vc*f[i1]*f[i2]    # uncomment for a smooth form factor
es,U = la.eigh(ham)    # U gives the orbital wave functions
#print es
e2=Ne*de
ev = es[Ne]
shift = (es[Ne]-Ne*de)/de
print 'fractional shift',shift
Egs = 0.0
for i in range(Ne):
  Egs += es[i]
print 'Egs', Egs
W0=array([[0.0]*(Ne+1)]*Nb)
sea0=W0[:,:-1]
for i in range(Ne) :
  W0[i,i] = 1.0
  sea0[i,i] = 1.0
s1=0.0
for i in range(Ne,Nb) :
  W0[i,Ne] = (wt[i])**0.5
  s1 += W0[i,Ne]
Wv0=dot(transpose(U),W0)
seav0 = dot(transpose(U),sea0)
M = dot(transpose(sea0),seav0)
print 'det(M)**2', la.det(M)**2
Wvt=array([[0.0j]*(Ne+1)]*Nb )
sea_olp = array([0.0j]*Nt)
sea_olp[0] = 1.0
G = dot(transpose(Wv0),Wv0)*de
#print 'G(0)',G
T=dt*Nt
f2 = open('Gt.dat','w')
ss = 'Nb,Ne,vc %i %i %8.6f\n' %(Nb,Ne,vc)
f2.write(ss) 
for it in range(0,Nt):
 t=dt*it
 for i1 in range(Nb) :
   Wvt[i1,:] = exp(-1.0j *es[i1]*t)*Wv0[i1,:]
 Msea = dot(transpose(seav0),Wvt[:,:-1])  
 sea_olp[it] = la.det(Msea)*exp(1.0j*Egs*t)
# olpm=dot(transpose(Wv0),Wvt)
# gt[it] =la.det(olpm)
 ss= ' %8.5f %8.5f %8.5f\n' %(t,sea_olp[it].real,sea_olp[it].imag)
 f2.write(ss)
# Fourier transform below
f2.close
Nres=8
sea_olp = conjugate(sea_olp)
fflag = 1
f1 = open('Gw.dat','w')
if fflag == 1:
  Ntp = Nt*Nres
  seaplus = array([0.0j]*Ntp)
  seaplus[0] = 0.5
  for it in range(1,Nt):
    x = dt*it/T 
    seaplus[it]=sea_olp[it]*(1.0 -3.0*x**2+2.0*x**3)
  sea_omega = fft.fft(seaplus)
  sea_omega = fft.fftshift(sea_omega)
  print  ' omega   Re(so(w))  Im(so(w) '
  for io in range(Ntp) :
    w= (io-Ntp/2.)*2*pi/(dt*Ntp)
    ss=' %7.4f  %8.5f  %8.5f\n ' % (w,real(sea_omega[io]),imag(sea_omega[io]))
#    print ss,
    f1.write(ss)
f1.close()
plot = 1
if plot == 1 :   # plot flag
  ss='''
set term postscript eps enhanced "Times-Roman" 28
set output "Gw.eps"
set nokey
set size 1.0,1.0
set xlabel " {/Symbol w}   "
set ylabel "  G'({/Symbol w})"
#set logscale x
#set logscale y
plot [-0.1:1.5] 'Gw.dat' using 1:2 w p,  'zero.dat' u 1:2 w l'''
  f2=open('Gw.gnu','w')
  f2.write(ss)
  f2.close()
  os.system('gnuplot Gw.gnu')
#  os.system('open Gw.eps')            
